#----------------------------------------------------------------------SDC
set SDC(frame-name) SDC
set SDC(frame-title) "Setting 2d Contours"
set SDC(icon-title)  optSDC

set preferences(SDC.base) 1
set preferences(SDC.mode) "linear"

set SDCmodes [list linear log custom]

set MAXCONTOURS 31

set preferences(SDC.contours) {}
for {set i 0} {$i < $MAXCONTOURS} {incr i} {set SDC(c$i) $i; lappend preferences(SDC.contours) $SDC(c$i)}

proc SDC.make-frame {} {
   global env preferences MAXCONTOURS SDC SDCmodes 

   SDC.unpack

  :frame $SDC(frame-name) :label "$SDC(frame-title)" :show-footer F :toplevel T \
        :icon-image "@$env(ICONHOME)/spectrum"  \
           :icon-name $SDC(icon-title) :resize F

   :panel SDC-PANEL :layout T
     :layout H

     set c1 20

     :canvas SDC-CANVAS :w 300 :h [expr [expr $MAXCONTOURS + 1] * $c1 + 10]
     for {set i 0} {$i < $MAXCONTOURS} {incr i} {
       set y1 [expr [expr $MAXCONTOURS - $i] * $c1]
       set y2 [expr $y1 + 15]
       :draw rectangle :tag colour$i :colour [lindex $preferences(CM.colours) $i] :value 30 $y1 130 $y2
     }

     for {set i 0} {$i < $MAXCONTOURS} {incr i} {
       set y3 [expr [expr $MAXCONTOURS - $i] * $c1]

       label [:path SDC-CANVAS].l$i -text "$i"   -width 2
       entry [:path SDC-CANVAS].e$i -relief flat -width 8 -justify right
       [:path SDC-CANVAS].e$i insert 0 $SDC(c$i)

       :draw window          :value 160 $y3 -window [:path SDC-CANVAS].l$i
       :draw window :tag e$i :value 230 $y3 -window [:path SDC-CANVAS].e$i 

       bind [:path SDC-CANVAS].e$i <KeyPress-Return> "set-contour $i"
    }

   :panel SDC-BUTTONS :layout T
     :layout H
     :button SDC-RESET   :label Reset   :command "SDC.reset-contours"
     :hskip 40
     :text   SDC-BASE    :label Base:   :value $preferences(SDC.base) :w 8
     :menubutton SDC-MODE :label "$preferences(SDC.mode)"   :strings $SDCmodes

     :next-row T
     :button SDC-SAVE    :label "Save preferences"   :command update_preferences
     :button SDC-UPDATE  :label Update :command SDC.Update
     :button SDC-HELP    :label Help    :command "click-help SDC"

   :show T
}

proc click-sdc-mode {w v} {
   global preferences SDCmodes
   set preferences(SDC.mode) [lindex $SDCmodes $v]
   :modify SDC-MODE :label "$preferences(SDC.mode)"
   SDC.announce
}

proc set-linear-contours b {
   global SDC MAXCONTOURS
   for {set i 0} {$i<$MAXCONTOURS} {incr i} {
      set SDC(c$i) [expr $b*$i]
   }
}

proc set-log-contours b {
   global SDC MAXCONTOURS
   set SDC(c0) 0
   set c 1
   for {set i 1} {$i<$MAXCONTOURS} {incr i} {
      set SDC(c$i) $c; set c [expr $c*$b]
   }
}

proc click-sdc-base {b} {
   global preferences
   set preferences(SDC.base) $b
   :modify SDC-BASE :value $b
   SDC.announce
}

proc set-contour {c} {
   global preferences SDC 
   set v [[:path SDC-CANVAS].e$c get]
   set vv $SDC(c$c)
   set SDC(c$c) $v
   if {$v>$vv} then normalise-contours-up
   if {$v<$vv} then normalise-contours-down
   set preferences(SDC.mode) Custom
   :modify SDC-MODE :label "$preferences(SDC.mode)"
   SDC.announce
}

proc normalise-contours-up {} {
   global SDC MAXCONTOURS
   set dirty 0
   set i 0
   set j 1
   while {$j<$MAXCONTOURS} {
      if {$SDC(c$i)>$SDC(c$j)} then {set SDC(c$j) $SDC(c$i); set dirty 1}
      incr i
      incr j
   }
   return $dirty
}

proc normalise-contours-down {} {
   global SDC MAXCONTOURS
   set dirty 0
   set i [expr $MAXCONTOURS - 1]
   set j [expr $MAXCONTOURS - 2]
   while {$j>=0} {
      if {$SDC(c$i)<$SDC(c$j)} then {set SDC(c$j) $SDC(c$i); set dirty 1}
      incr i -1
      incr j -1
   }
   return $dirty
}

proc SDC.reset-contours {} {
   global preferences SDC
#   set preferences(SDC.base) 1
   :modify SDC-BASE :value $preferences(SDC.base)
#   set preferences(SDC.mode) linear
   :modify SDC-MODE :label "$preferences(SDC.mode)"
   SDC.announce
}

proc SDC.unpack {} {
   global preferences SDC

   if {"$preferences(SDC.mode)" != "linear" && "$preferences(SDC.mode)" != "log" } then {set preferences(SDC.mode) "custom"}

   set i 0
   foreach c $preferences(SDC.contours) {
      set SDC(c$i) $c
      incr i
   }
}

proc SDC.announce {} {
   global preferences SDC MAXCONTOURS
   if {$preferences(SDC.mode) == "linear"} then {set-linear-contours $preferences(SDC.base)}
   if {$preferences(SDC.mode) == "log"}    then {set-log-contours $preferences(SDC.base)}

   set preferences(SDC.contours) {}
   for {set i 0} {$i < $MAXCONTOURS} {incr i} {lappend preferences(SDC.contours) $SDC(c$i)}

   inform sd set-contour-levels

   for {set i 0} {$i<$MAXCONTOURS} {incr i 1} {
     [:path SDC-CANVAS].e$i delete 0 end
     [:path SDC-CANVAS].e$i insert 0 $SDC(c$i)
   }
}

proc SDC.save {} {}

proc SDC.Update {} {
   global preferences MAXCOLOURS MAXCONTOURS

   for {set i 0} {$i<$MAXCOLOURS} {incr i 1} {
     [:path SDC-CANVAS] itemconfigure colour$i -fill [lindex $preferences(CM.colours) $i]
   }

   for {set i 0} {$i<$MAXCONTOURS} {incr i 1} {
     [:path SDC-CANVAS].e$i delete 0 end
     [:path SDC-CANVAS].e$i insert 0 [lindex $preferences(SDC.contours) $i]
   }
}
 
